@@ -6,6 +6,10 @@ class Formatter:
 
     def __init__(self, tokens):
         self.__tokens = tokens
+        self.__namings_to_fix = dict()
+
+    def get_tokens(self):
+        return self.__tokens
 
     @staticmethod
     def snake_to_lower_camel_case(naming):
@@ -14,7 +18,8 @@ class Formatter:
         if naming.find('_') != -1 and naming[0].isupper():
             return naming[0].lower() + naming[1:]
         components = naming.split('_')
-        return components[0].lower() + ''.join(x[0].capitalize() + x[1:] for x in components[1:])
+        return (components[0][0].lower() + components[0][1:]) + ''.join(
+            x[0].capitalize() + x[1:] for x in components[1:])
 
     @staticmethod
     def __is_not_lower_camel_case(naming):
@@ -34,13 +39,12 @@ class Formatter:
 
     @staticmethod
     def snake_to_upper_camel_case(naming):
-        return Formatter.snake_to_lower_camel_case(naming).title()
+        lower_camel_case = Formatter.snake_to_lower_camel_case(naming)
+        return lower_camel_case[0].upper() + lower_camel_case[1:]
 
-    def format(self):
+    def fix_namings(self, namings_to_fix):
         self.__fix_comments()
-        self.__fix_namings()
-        self.__fix_type_params()
-        return self.__tokens
+        self.__fix_namings(namings_to_fix)
 
     def fix_filename(self, filename):
         # upper case camel case for files with single logical compilation unit
@@ -59,30 +63,18 @@ class Formatter:
             if self.__tokens[i].get_value() in Formatter.class_keywords:
                 i = self.__skip_ws_tokens(i + 1)
                 if self.__tokens[i].get_value() not in class_names:
-                    class_names.append(self.__tokens.get_value())
+                    class_names.append(self.__tokens[i].get_value())
                     count += 1
-                i += 1
+            i += 1
         return count
 
-    def __fix_namings(self):
+    def find_namings_to_fix(self):
+        self.__fix_method_variables()
         i = 0
         while i < len(self.__tokens):
             token = self.__tokens[i]
-            # upper case camel case for object types
-            if token.get_value() == ':':
-                i = self.__skip_ws_tokens(i + 1)
-                if self.__tokens[i].get_token_type() == TokenType.IDENTIFIER:
-                    if Formatter.__is_not_upper_camel_case(self.__tokens[i].get_value()):
-                        self.__fix_similar(self.__tokens[i].get_primary_value(),
-                                           Formatter.snake_to_upper_camel_case(self.__tokens[i].get_value()))
-                    i += 1
-            # lower case camel case for annotation
-            elif token.get_token_type() == TokenType.ANNOTATION and Formatter.__is_not_lower_camel_case(
-                    token.get_value()[1:]):
-                token.set_fixed_value('@' + Formatter.snake_to_lower_camel_case(token.get_value()[1:]))
-                i += 1
             # package as in Java naming convention
-            elif token.get_value() == 'package':
+            if token.get_value() == 'package':
                 i = self.__fix_packages(i) - 1
             # upper case camel case for class, trait & object
             elif token.get_value() == 'extends':
@@ -97,33 +89,14 @@ class Formatter:
                 while self.__tokens[i].get_token_type() != TokenType.IDENTIFIER:
                     i += 1
                 if Formatter.__is_not_lower_camel_case(self.__tokens[i].get_value()):
-                    self.__fix_similar(self.__tokens[i].get_primary_value(),
-                                       Formatter.snake_to_lower_camel_case(self.__tokens[i].get_value()))
-                i += 1
-            elif token.get_token_type() == TokenType.IDENTIFIER and token.get_fixed_value() is None:
-                pos = i + 1
-                while self.__tokens[pos].get_value() == ' ':
-                    pos += 1
-                if self.__tokens[pos].get_value() not in (
-                        '.', '[', 'extends', '(', '{') and Formatter.__is_not_lower_camel_case(
-                    token.get_value()):
-
-                    self.__fix_similar(token.get_primary_value(),
-                                       Formatter.snake_to_lower_camel_case(token.get_value()))
+                    self.__namings_to_fix[self.__tokens[i].get_primary_value()] = Formatter.snake_to_lower_camel_case(
+                        self.__tokens[i].get_value())
+                i = self.__fix_generic_if_exists(i)
                 i += 1
             else:
                 i += 1
 
-    def __fix_type_params(self):
-        brace_count = 0
-        for token in self.__tokens:
-            if token.get_value() == '[':
-                brace_count += 1
-            elif token.get_value() == ']':
-                brace_count -= 1
-            elif brace_count > 0 and token.get_token_type() == TokenType.IDENTIFIER and Formatter.__is_not_upper_camel_case(
-                    token.get_value()):
-                self.__fix_similar(token.get_primary_value(), Formatter.snake_to_upper_camel_case(token.get_value()))
+        return self.__namings_to_fix
 
     def __fix_packages(self, pos):
         pos = self.__skip_ws_tokens(pos + 1)
@@ -137,11 +110,25 @@ class Formatter:
 
     def __fix_class_trait_object(self, pos):
         pos = self.__skip_ws_tokens(pos + 1)
-        if self.__tokens[pos].get_token_type() == TokenType.IDENTIFIER and Formatter.__is_not_upper_camel_case(
-                self.__tokens[pos].get_value()):
-            fixed_value = Formatter.snake_to_upper_camel_case(self.__tokens[pos].get_value())
-            self.__fix_similar(self.__tokens[pos].get_primary_value(), fixed_value)
-            pos += 1
+        if self.__tokens[pos].get_token_type() == TokenType.IDENTIFIER:
+            if Formatter.__is_not_upper_camel_case(
+                    self.__tokens[pos].get_value()):
+                fixed_value = Formatter.snake_to_upper_camel_case(self.__tokens[pos].get_value())
+                self.__namings_to_fix[self.__tokens[pos].get_primary_value()] = fixed_value
+            pos = self.__fix_generic_if_exists(pos)
+        self.__fix_class_params(pos)
+        return pos
+
+    def __fix_generic_if_exists(self, pos):
+        pos = self.__skip_ws_tokens(pos + 1)
+        if self.__tokens[pos].get_value() == '[':
+            pos = self.__skip_ws_tokens(pos + 1)
+            if Formatter.__is_not_upper_camel_case(self.__tokens[pos].get_value()):
+                self.__namings_to_fix[self.__tokens[pos].get_value()] = Formatter.snake_to_upper_camel_case(
+                    self.__tokens[pos].get_value())
+            while self.__tokens[pos].get_value() != ']':
+                pos += 1
+        pos += 1
         return pos
 
     def __fix_comments(self):
@@ -427,6 +414,66 @@ class Formatter:
 
         return comment[:start] + comment[end:]
 
+    def __fix_method_variables(self):
+        i = 0
+        while i < len(self.__tokens):
+            if self.__tokens[i].get_value() == 'def':
+                self.__fix_method_params(i+1)
+            i += 1
+
+    def __fix_method_params(self, pos):
+        params = self.__find_params(pos)
+        if len(params) > 0:
+            brace_found = False
+            brace_count = 0
+            while pos < len(self.__tokens) and self.__tokens[pos].get_value() not in Formatter.class_keywords and \
+                    not (self.__tokens[pos].get_value() in ('def', 'val', 'var') and brace_count == 0) and not (
+                    brace_found and brace_count == -1):
+                if self.__tokens[pos].get_value() in params and Formatter.__is_not_lower_camel_case(
+                        self.__tokens[pos].get_value()):
+                    self.__tokens[pos].set_fixed_value(
+                        Formatter.snake_to_lower_camel_case(self.__tokens[pos].get_value()))
+                elif self.__tokens[pos].get_value() == '{':
+                    brace_found = True
+                    brace_count += 1
+                elif self.__tokens[pos].get_value() == '}':
+                    brace_count -= 1
+                elif self.__tokens[pos].get_value() in ['def', 'val', 'var']:
+                    current = self.__tokens[pos].get_value()
+                    pos = self.__skip_ws_tokens(pos + 1)
+                    if Formatter.__is_not_lower_camel_case(self.__tokens[pos].get_value()):
+                        self.__tokens[pos].set_fixed_value(Formatter.snake_to_lower_camel_case(self.__tokens[pos].get_value()))
+                        params.append(self.__tokens[pos].get_value())
+                    elif current == 'val':
+                        self.__tokens[pos].set_fixed_value(self.__tokens[pos].get_value())
+                pos += 1
+
+    def __fix_class_params(self, pos):
+        params = self.__find_params(pos)
+        brace_found = False
+        brace_count = 0
+        while not (brace_found and brace_count == 0):
+            if self.__tokens[pos].get_value() in params and Formatter.__is_not_lower_camel_case(self.__tokens[pos]):
+                self.__tokens[pos].set_fixed_value(
+                    Formatter.snake_to_lower_camel_case(self.__tokens[pos].get_value()))
+            elif self.__tokens[pos].get_value() == '{':
+                brace_found = True
+                brace_count += 1
+            elif self.__tokens[pos].get_value() == '}':
+                brace_count -= 1
+            elif self.__tokens[pos].get_value() in ['def', 'val', 'var']:
+                current = self.__tokens[pos].get_value()
+                pos = self.__skip_ws_tokens(pos + 1)
+                if current == 'val' and self.__tokens[pos].get_fixed_value() is None:
+                    if Formatter.__is_not_upper_camel_case(self.__tokens[pos].get_value()):
+                        self.__namings_to_fix[self.__tokens[pos].get_value()] = Formatter.snake_to_upper_camel_case(
+                            self.__tokens[pos].get_value())
+                else:
+                    if Formatter.__is_not_lower_camel_case(self.__tokens[pos].get_value()):
+                        self.__namings_to_fix[self.__tokens[pos].get_value()] = Formatter.snake_to_lower_camel_case(
+                            self.__tokens[pos].get_value())
+            pos += 1
+
     def __fix_method_comment(self, pos, comment=None):
         params = self.__find_params(pos)
         throws = self.__find_throws(pos)
@@ -692,10 +739,9 @@ class Formatter:
                 i += 1
             return i
 
-    def __fix_similar(self, token_before, token_after):
-        i = 0
-        while i < len(self.__tokens):
-            token = self.__tokens[i]
-            if token.get_primary_value() == token_before:
-                token.set_fixed_value(token_after)
-            i += 1
+    def __fix_namings(self, namings_to_fix):
+        for token in self.__tokens:
+            if token.get_value() in namings_to_fix.keys():
+                token.set_fixed_value(namings_to_fix[token.get_value()])
+            elif token.get_token_type() == TokenType.ANNOTATION and token.get_value()[1:] in namings_to_fix.keys():
+                token.set_fixed_value('@' + namings_to_fix[token.get_value()[1:]])
